@@ -6,16 +6,16 @@ use Text::Xslate::Util qw(
     value_to_literal
 );
 
+
 use Carp ();
 use Scalar::Util ();
 
+use Text::Xslate::PP::Const;
 use Text::Xslate::Util qw(
     $DEBUG p neat
     value_to_literal
     mark_raw unmark_raw html_escape
 );
-use Text::Xslate::PP::Const;
-use Text::Xslate::PP::Booster;
 
 use constant _DUMP_PP => scalar($DEBUG =~ /\b dump=pp \b/xms);
 
@@ -60,11 +60,17 @@ has stash => ( is => 'rw', default => sub { {}; } ); # store misc data
 #
 # public APIs
 #
+
 sub opcode_to_perlcode {
     my ( $self, $opcode ) = @_;
 
     my $perlcode = $self->opcode_to_perlcode_string( $opcode );
-    return Text::Xslate::PP::Booster->compile($perlcode);
+
+    print STDERR "$perlcode\n" if _DUMP_PP;
+
+    package Text::Xslate::PP::Booster;
+
+    return eval($perlcode) || Carp::confess("Eval error: $@");
 }
 
 
@@ -97,7 +103,6 @@ CODE
     $perlcode .= join( '', grep { defined } @{ $self->{lines} } );
     $perlcode .= "\n" . '    return $output;' . "\n}";
 
-    print STDERR $perlcode, "\n" if _DUMP_PP;
     return $perlcode;
 }
 
@@ -308,7 +313,7 @@ $CODE_MANIP{ 'include' } = sub {
 # include
 {
     $st->{pc} = %2$s; # for error messages
-    my $st2 = Text::Xslate::PP::tx_load_template( $st->engine, %1$s, 1 );
+    my $st2 = Text::Xslate::PP::tx_load_template( $st->engine, %1$s );
     $output .= Text::Xslate::PP::tx_execute( $st2, $vars );
 }
 CODE
@@ -396,20 +401,7 @@ $CODE_MANIP{ 'div' } = sub {
 
 $CODE_MANIP{ 'mod' } = sub {
     my ( $self, $arg, $line ) = @_;
-    $self->sa( sprintf(<<'CODE', $self->sb(), $self->sa() ) );
-    do {
-        my $lhs = %s;
-        my $rhs = %s;
-        if($rhs == 0) {
-            $st->error(undef, "Illegal modulus zero");
-            'NaN';
-        }
-        else {
-            $lhs %% $rhs;
-        }
-    }
-CODE
-
+    $self->sa( sprintf( '( %s %% %s )', $self->sb(), $self->sa() ) );
     $self->optimize_to_print( 'num' );
 };
 
@@ -419,10 +411,6 @@ $CODE_MANIP{ 'concat' } = sub {
     $self->sa( sprintf( 'Text::Xslate::PP::tx_concat( %s, %s )', $self->sb(), $self->sa() ) );
 };
 
-$CODE_MANIP{ 'repeat' } = sub {
-    my ( $self, $arg, $line ) = @_;
-    $self->sa( sprintf( 'Text::Xslate::PP::tx_repeat( %s, %s )', $self->sb(), $self->sa() ) );
-};
 
 $CODE_MANIP{ 'bitor' } = sub {
     my ( $self, $arg, $line ) = @_;
@@ -560,11 +548,6 @@ $CODE_MANIP{ 'scmp' } = sub {
     $self->sa( sprintf( '( %s cmp %s )', _rm_tailed_lf( $self->sb() ), _rm_tailed_lf( $self->sa() ) ) );
 };
 
-$CODE_MANIP{ 'range' } = sub {
-    my ( $self, $arg, $line ) = @_;
-    push @{ $self->SP->[ -1 ] }, sprintf('( %s .. %s )',
-        _rm_tailed_lf( $self->sb() ), _rm_tailed_lf( $self->sa() ) );
-};
 
 $CODE_MANIP{ 'macrocall' } = sub {
     my ( $self, $arg, $line ) = @_;
@@ -1160,7 +1143,7 @@ __END__
 
 =head1 NAME
 
-Text::Xslate::PP::Compiler::CodeGenerator - An Xslate code generator for PP::Booster
+Text::Xslate::Compiler::PPBooster::CodeGenerator - PP::Booster code generator
 
 =head1 DESCRIPTION
 
